open Ast
open Parser
open Lexer
open Hammer
open Interpreter

open Format
open Arg
open Lexing

let usage = "usage: interp [--steps [--all] [--ask] | -i] file.hoc"

let iter = ref 0
let steps = ref false
let all = ref false
let ask = ref false

let specs =[
  "--steps", Set steps, "stops at each non-deterministic choice";
  "--all", Set all, "  stops after each transition";
  "--ask", Set ask, "  prompts the user to choose the desired transition";
  "-i", Set_int iter, "     Number of iterations"
  ]

let file =
  let f = ref None in
  let set_file s =
    if not (Filename.check_suffix s ".hoc")
    then raise (Arg.Bad "no .hoc extension");
    f := Some s in
  Arg.parse specs set_file usage;
  match !f with
  | None -> Arg.usage [] usage; exit 1
  | Some f -> f

let reportPos (b,e) =
  let l = b.pos_lnum in
  let fc = b.pos_cnum - b.pos_bol + 1 in
  let lc = e.pos_cnum - b.pos_bol + 1 in
  eprintf "File \"%s\", line %d, characters %d-%d:\n" file l fc lc


let () =
  Random.self_init ();
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let f = Parser.init Lexer.token lb in
    close_in c;
    let p = Hammer.hammer f in
    if !steps then begin
      if !iter <> 0 then eprintf "Warning: -i parameter useless with -steps@.";
      if !all then begin
        if !ask then (printf "@.------@.@."; Interpreter.machine_ask_all p)
        else (printf "@.------@.@."; Interpreter.machine_small_steps p)
      end
      else if !ask then (printf "@.------@.@."; Interpreter.machine_ask p)
      else (printf "@.------@.@."; Interpreter.machine_steps p)
    end
    else if !all || !ask then failwith "Specify --steps to show the steps"
    else if !iter = 0
    then failwith "Specify a maximum number of reductions with -i"
    else (printf "@.------@.@."; Interpreter.machine_random !iter p);
    exit 0
  with
  | Lexer.LexerError s ->
    close_in c;
    reportPos  (lexeme_start_p lb, lexeme_end_p lb);
    eprintf "Lexical error: %s@." s;
    exit 1
  | Parser.Error ->
    close_in c;
    reportPos  (lexeme_start_p lb, lexeme_end_p lb);
    eprintf "Syntax error@.";
    exit 1
  | Failure x ->
    eprintf "Error: %s@." x;
    exit 1
  | e ->
    close_in c;
    eprintf "Anomaly: %s@." (Printexc.to_string e);
    exit 2
